@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2014 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -47,6 +47,8 @@ from Mailman.CSRFcheck import csrf_check
 # Set up i18n
 _ = i18n._
 i18n.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
+def D_(s):
+    return s
 
 NL = '\n'
 OPTCOLUMNS = 11
@@ -86,6 +88,18 @@ def main():
     i18n.set_language(mlist.preferred_language)
     # If the user is not authenticated, we're done.
     cgidata = cgi.FieldStorage(keep_blank_values=1)
+    try:
+        cgidata.getfirst('csrf_token', '')
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc = Document()
+        doc.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
 
     # CSRF check
     safe_params = ['VARHELP', 'adminpw', 'admlogin',
@@ -93,20 +107,27 @@ def main():
                    'legend']
     params = cgidata.keys()
     if set(params) - set(safe_params):
-        csrf_checked = csrf_check(mlist, cgidata.getvalue('csrf_token'))
+        csrf_checked = csrf_check(mlist, cgidata.getfirst('csrf_token'))
     else:
         csrf_checked = True
     # if password is present, void cookie to force password authentication.
-    if cgidata.getvalue('adminpw'):
+    if cgidata.getfirst('adminpw'):
         os.environ['HTTP_COOKIE'] = ''
         csrf_checked = True
 
     if not mlist.WebAuthenticate((mm_cfg.AuthListAdmin,
                                   mm_cfg.AuthSiteAdmin),
-                                 cgidata.getvalue('adminpw', '')):
+                                 cgidata.getfirst('adminpw', '')):
         if cgidata.has_key('adminpw'):
             # This is a re-authorization attempt
             msg = Bold(FontSize('+1', _('Authorization failed.'))).Format()
+            remote = os.environ.get('HTTP_FORWARDED_FOR',
+                     os.environ.get('HTTP_X_FORWARDED_FOR',
+                     os.environ.get('REMOTE_ADDR',
+                                    'unidentified origin')))
+            syslog('security',
+                   'Authorization failed (admin): list=%s: remote=%s',
+                   listname, remote)
         else:
             msg = ''
         Auth.loginpage(mlist, 'admin', msg=msg)
@@ -143,7 +164,7 @@ def main():
     if qsenviron:
         parsedqs = cgi.parse_qs(qsenviron)
     if cgidata.has_key('VARHELP'):
-        varhelp = cgidata.getvalue('VARHELP')
+        varhelp = cgidata.getfirst('VARHELP')
     elif parsedqs:
         # POST methods, even if their actions have a query string, don't get
         # put into FieldStorage's keys :-(
@@ -260,7 +281,11 @@ def admin_overview(msg=''):
     listnames.sort()
 
     for name in listnames:
-        mlist = MailList.MailList(name, lock=0)
+        try:
+            mlist = MailList.MailList(name, lock=0)
+        except Errors.MMUnknownListError:
+            # The list could have been deleted by another process.
+            continue
         if mlist.advertised:
             if mm_cfg.VIRTUAL_HOST_OVERVIEW and (
                    mlist.web_page_url.find('/%s/' % hostname) == -1 and
@@ -270,7 +295,7 @@ def admin_overview(msg=''):
             else:
                 advertised.append((mlist.GetScriptURL('admin'),
                                    mlist.real_name,
-                                   mlist.description))
+                                   Utils.websafe(mlist.GetDescription())))
     # Greeting depends on whether there was an error or not
     if msg:
         greeting = FontAttr(msg, color="ff5060", size="+1")
@@ -892,12 +917,12 @@ def membership_options(mlist, subcat, cgidata, doc, form):
     container.AddItem(header)
     # Add a "search for member" button
     table = Table(width='100%')
-    link = Link('http://docs.python.org/library/re.html'
+    link = Link('https://docs.python.org/2/library/re.html'
                 '#regular-expression-syntax',
                 _('(help)')).Format()
     table.AddRow([Label(_('Find member %(link)s:')),
                   TextBox('findmember',
-                          value=cgidata.getvalue('findmember', '')),
+                          value=cgidata.getfirst('findmember', '')),
                   SubmitButton('findmember_btn', _('Search...'))])
     container.AddItem(table)
     container.AddItem('<hr><p>')
@@ -910,7 +935,16 @@ def membership_options(mlist, subcat, cgidata, doc, form):
     all = [_m.encode() for _m in mlist.getMembers()]
     all.sort(lambda x, y: cmp(x.lower(), y.lower()))
     # See if the query has a regular expression
-    regexp = cgidata.getvalue('findmember', '').strip()
+    regexp = cgidata.getfirst('findmember', '').strip()
+    try:
+        regexp = regexp.decode(Utils.GetCharSet(mlist.preferred_language))
+    except UnicodeDecodeError:
+        # This is probably a non-ascii character and an English language
+        # (ascii) list.  Even if we didn't throw the UnicodeDecodeError,
+        # the input may have contained mnemonic or numeric HTML entites mixed
+        # with other characters.  Trying to grok the real meaning out of that
+        # is complex and error prone, so we don't try.
+        pass
     if regexp:
         try:
             cre = re.compile(regexp, re.IGNORECASE)
@@ -985,6 +1019,9 @@ def membership_options(mlist, subcat, cgidata, doc, form):
             if regexp:
                 findfrag = '&findmember=' + urllib.quote(regexp)
             url = adminurl + '/members?letter=' + letter + findfrag
+            if isinstance(url, unicode):
+                url = url.encode(Utils.GetCharSet(mlist.preferred_language),
+                                 errors='ignore')
             if letter == bucket:
                 show = Bold('[%s]' % letter.upper()).Format()
             else:
@@ -1027,7 +1064,8 @@ def membership_options(mlist, subcat, cgidata, doc, form):
         fullname = Utils.uncanonstr(mlist.getMemberName(addr),
                                     mlist.preferred_language)
         name = TextBox(qaddr + '_realname', fullname, size=longest).Format()
-        cells = [Center(CheckBox(qaddr + '_unsub', 'off', 0).Format()),
+        cells = [Center(CheckBox(qaddr + '_unsub', 'off', 0).Format()
+                        + '<div class="hidden">' + _('unsub') + '</div>'),
                  link.Format() + '<br>' +
                  name +
                  Hidden('user', qaddr).Format(),
@@ -1040,9 +1078,12 @@ def membership_options(mlist, subcat, cgidata, doc, form):
             value = 'off'
             checked = 0
         box = CheckBox('%s_mod' % qaddr, value, checked)
-        cells.append(Center(box).Format())
+        cells.append(Center(box.Format()
+            + '<div class="hidden">' + _('mod') + '</div>'))
+        # Kluge, get these translated.
+        (_('hide'), _('nomail'), _('ack'), _('notmetoo'), _('nodupes'))
         for opt in ('hide', 'nomail', 'ack', 'notmetoo', 'nodupes'):
-            extra = ''
+            extra = '<div class="hidden">' + _(opt) + '</div>'
             if opt == 'nomail':
                 status = mlist.getDeliveryStatus(addr)
                 if status == MemberAdaptor.ENABLED:
@@ -1051,7 +1092,7 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                 else:
                     value = 'on'
                     checked = 1
-                    extra = '[%s]' % ds_abbrevs[status]
+                    extra = '[%s]' % ds_abbrevs[status] + extra
             elif mlist.getMemberOption(addr, mm_cfg.OPTINFO[opt]):
                 value = 'on'
                 checked = 1
@@ -1064,17 +1105,22 @@ def membership_options(mlist, subcat, cgidata, doc, form):
         # the underlying dictionary attribute.  This version is slower and
         # less memory efficient.  It points to a new MemberAdaptor interface
         # method.
+        extra = '<div class="hidden">' + _('digest') + '</div>'
         if addr in mlist.getRegularMemberKeys():
-            cells.append(Center(CheckBox(qaddr + '_digest', 'off', 0).Format()))
+            cells.append(Center(CheckBox(qaddr + '_digest', 'off', 0).Format()
+                                + extra))
         else:
-            cells.append(Center(CheckBox(qaddr + '_digest', 'on', 1).Format()))
+            cells.append(Center(CheckBox(qaddr + '_digest', 'on', 1).Format()
+                                + extra))
         if mlist.getMemberOption(addr, mm_cfg.OPTINFO['plain']):
             value = 'on'
             checked = 1
         else:
             value = 'off'
             checked = 0
-        cells.append(Center(CheckBox('%s_plain' % qaddr, value, checked)))
+        cells.append(Center(CheckBox(
+                            '%s_plain' % qaddr, value, checked).Format()
+                            + '<div class="hidden">' + _('plain') + '</div>'))
         # User's preferred language
         langpref = mlist.getMemberLanguage(addr)
         langs = mlist.GetAvailableLanguages()
@@ -1160,8 +1206,12 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                 continue
             start = chunkmembers[i*chunksz]
             end = chunkmembers[min((i+1)*chunksz, last)-1]
-            link = Link(url + 'chunk=%d' % i + findfrag,
-                        _('from %(start)s to %(end)s'))
+            thisurl = url + 'chunk=%d' % i + findfrag
+            if isinstance(thisurl, unicode):
+                thisurl = thisurl.encode(
+                                 Utils.GetCharSet(mlist.preferred_language),
+                                 errors='ignore')
+            link = Link(thisurl, _('from %(start)s to %(end)s'))
             buttons.append(link)
         buttons = UnorderedList(*buttons)
         container.AddItem(footer + buttons.Format() + '<p>')
@@ -1183,7 +1233,7 @@ def mass_subscribe(mlist, container):
     table.AddCellInfo(table.GetCurrentRowIndex(), 0, bgcolor=GREY)
     table.AddCellInfo(table.GetCurrentRowIndex(), 1, bgcolor=GREY)
     table.AddRow([
-        Label(_('Send welcome messages to new subscribees?')),
+        Label(_('Send welcome messages to new subscribers?')),
         RadioButtonArray('send_welcome_msg_to_this_batch',
                          (_('No'), _('Yes')),
                          mlist.send_welcome_msg,
@@ -1351,16 +1401,17 @@ def submit_button(name='submit'):
 
 
 def change_options(mlist, category, subcat, cgidata, doc):
+    global _
     def safeint(formvar, defaultval=None):
         try:
-            return int(cgidata.getvalue(formvar))
+            return int(cgidata.getfirst(formvar))
         except (ValueError, TypeError):
             return defaultval
     confirmed = 0
     # Handle changes to the list moderator password.  Do this before checking
     # the new admin password, since the latter will force a reauthentication.
-    new = cgidata.getvalue('newmodpw', '').strip()
-    confirm = cgidata.getvalue('confirmmodpw', '').strip()
+    new = cgidata.getfirst('newmodpw', '').strip()
+    confirm = cgidata.getfirst('confirmmodpw', '').strip()
     if new or confirm:
         if new == confirm:
             mlist.mod_password = sha_new(new).hexdigest()
@@ -1370,8 +1421,8 @@ def change_options(mlist, category, subcat, cgidata, doc):
             doc.addError(_('Moderator passwords did not match'))
     # Handle changes to the list poster password.  Do this before checking
     # the new admin password, since the latter will force a reauthentication.
-    new = cgidata.getvalue('newpostpw', '').strip()
-    confirm = cgidata.getvalue('confirmpostpw', '').strip()
+    new = cgidata.getfirst('newpostpw', '').strip()
+    confirm = cgidata.getfirst('confirmpostpw', '').strip()
     if new or confirm:
         if new == confirm:
             mlist.post_password = sha_new(new).hexdigest()
@@ -1380,8 +1431,8 @@ def change_options(mlist, category, subcat, cgidata, doc):
         else:
             doc.addError(_('Poster passwords did not match'))
     # Handle changes to the list administrator password
-    new = cgidata.getvalue('newpw', '').strip()
-    confirm = cgidata.getvalue('confirmpw', '').strip()
+    new = cgidata.getfirst('newpw', '').strip()
+    confirm = cgidata.getfirst('confirmpw', '').strip()
     if new or confirm:
         if new == confirm:
             mlist.password = sha_new(new).hexdigest()
@@ -1397,8 +1448,8 @@ def change_options(mlist, category, subcat, cgidata, doc):
         gui.handleForm(mlist, category, subcat, cgidata, doc)
     # mass subscription, removal processing for members category
     subscribers = ''
-    subscribers += cgidata.getvalue('subscribees', '')
-    subscribers += cgidata.getvalue('subscribees_upload', '')
+    subscribers += cgidata.getfirst('subscribees', '')
+    subscribers += cgidata.getfirst('subscribees_upload', '')
     if subscribers:
         entries = filter(None, [n.strip() for n in subscribers.splitlines()])
         send_welcome_msg = safeint('send_welcome_msg_to_this_batch',
@@ -1407,7 +1458,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
                                    mlist.admin_notify_mchanges)
         # Default is to subscribe
         subscribe_or_invite = safeint('subscribe_or_invite', 0)
-        invitation = cgidata.getvalue('invitation', '')
+        invitation = cgidata.getfirst('invitation', '')
         digest = mlist.digest_is_default
         if not mlist.digestable:
             digest = 0
@@ -1434,9 +1485,12 @@ def change_options(mlist, category, subcat, cgidata, doc):
                     else:
                         mlist.InviteNewMember(userdesc, invitation)
                 else:
+                    _ = D_
+                    whence = _('admin mass sub')
+                    _ = i18n._
                     mlist.ApprovedAddMember(userdesc, send_welcome_msg,
                                             send_admin_notif, invitation,
-                                            whence='admin mass sub')
+                                            whence=whence)
             except Errors.MMAlreadyAMember:
                 subscribe_errors.append((safeentry, _('Already a member')))
             except Errors.MMBadEmailError:
@@ -1489,8 +1543,11 @@ def change_options(mlist, category, subcat, cgidata, doc):
         unsubscribe_success = []
         for addr in names:
             try:
+                _ = D_
+                whence = _('admin mass unsub')
+                _ = i18n._
                 mlist.ApprovedDeleteMember(
-                    addr, whence='admin mass unsub',
+                    addr, whence=whence,
                     admin_notif=send_unsub_notifications,
                     userack=userack)
                 unsubscribe_success.append(Utils.websafe(addr))
@@ -1508,8 +1565,8 @@ def change_options(mlist, category, subcat, cgidata, doc):
             doc.AddItem('<p>')
     # Address Changes
     if cgidata.has_key('change_from'):
-        change_from = cgidata.getvalue('change_from', '')
-        change_to = cgidata.getvalue('change_to', '')
+        change_from = cgidata.getfirst('change_from', '')
+        change_to = cgidata.getfirst('change_to', '')
         schange_from = Utils.websafe(change_from)
         schange_to = Utils.websafe(change_to)
         success = False
@@ -1553,7 +1610,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
 """))
         subject = _('%(list_name)s address change notice.')
         i18n.set_translation(otrans)
-        if success and cgidata.getvalue('notice_old', '') == 'yes':
+        if success and cgidata.getfirst('notice_old', '') == 'yes':
             # Send notice to old address.
             msg = Message.UserNotification(change_from,
                 mlist.GetOwnerEmail(),
@@ -1563,7 +1620,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
                 )
             msg.send(mlist)
             doc.AddItem(Header(3, _('Notification sent to %(schange_from)s.')))
-        if success and cgidata.getvalue('notice_new', '') == 'yes':
+        if success and cgidata.getfirst('notice_new', '') == 'yes':
             # Send notice to new address.
             msg = Message.UserNotification(change_to,
                 mlist.GetOwnerEmail(),
@@ -1597,7 +1654,10 @@ def change_options(mlist, category, subcat, cgidata, doc):
             quser = urllib.quote(user)
             if cgidata.has_key('%s_unsub' % quser):
                 try:
-                    mlist.ApprovedDeleteMember(user, whence='member mgt page')
+                    _ = D_
+                    whence=_('member mgt page')
+                    _ = i18n._
+                    mlist.ApprovedDeleteMember(user, whence=whence)
                     removes.append(user)
                 except Errors.NotAMemberError:
                     errors.append((user, _('Not subscribed')))
@@ -1616,16 +1676,16 @@ def change_options(mlist, category, subcat, cgidata, doc):
                 # BAW: Hmm...
                 pass
 
-            newname = cgidata.getvalue(quser+'_realname', '')
+            newname = cgidata.getfirst(quser+'_realname', '')
             newname = Utils.canonstr(newname, mlist.preferred_language)
             mlist.setMemberName(user, newname)
 
-            newlang = cgidata.getvalue(quser+'_language')
+            newlang = cgidata.getfirst(quser+'_language')
             oldlang = mlist.getMemberLanguage(user)
             if Utils.IsLanguage(newlang) and newlang <> oldlang:
                 mlist.setMemberLanguage(user, newlang)
 
-            moderate = not not cgidata.getvalue(quser+'_mod')
+            moderate = not not cgidata.getfirst(quser+'_mod')
             mlist.setMemberOption(user, mm_cfg.Moderate, moderate)
 
             # Set the `nomail' flag, but only if the user isn't already
