@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2014 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -255,7 +255,7 @@ class General(GUIBase):
              their own <tt>Reply-To:</tt> settings to convey their valid
              return address.  Another is that modifying <tt>Reply-To:</tt>
              makes it much more difficult to send private replies.  See <a
-             href="http://www.unicom.com/pw/reply-to-harmful.html">`Reply-To'
+             href="http://marc.merlins.org/netrants/reply-to-harmful.html">`Reply-To'
              Munging Considered Harmful</a> for a general discussion of this
              issue.  See <a
              href="http://marc.merlins.org/netrants/reply-to-useful.html">
@@ -283,7 +283,7 @@ class General(GUIBase):
              their own <tt>Reply-To:</tt> settings to convey their valid
              return address.  Another is that modifying <tt>Reply-To:</tt>
              makes it much more difficult to send private replies.  See <a
-             href="http://www.unicom.com/pw/reply-to-harmful.html">`Reply-To'
+             href="http://marc.merlins.org/netrants/reply-to-harmful.html">`Reply-To'
              Munging Considered Harmful</a> for a general discussion of this
              issue.  See <a
              href="http://marc.merlins.org/netrants/reply-to-useful.html">
@@ -559,6 +559,14 @@ mlist.info.
                                           or not isinstance(val, IntType)):
             doc.addError(_("""<b>admin_member_chunksize</b> attribute not
             changed!  It must be an integer > 0."""))
+        elif property == 'host_name':
+            try:
+                Utils.ValidateEmail('user@' + val)
+            except Errors.EmailAddressError:
+                doc.addError(_("""<b>host_name</b> attribute not changed!
+                It must be a valid domain name."""))
+            else:
+                GUIBase._setValue(self, mlist, property, val, doc)
         else:
             GUIBase._setValue(self, mlist, property, val, doc)
 
