@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2014 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +15,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 # USA.
 
-"""Cook a message's Subject header."""
+"""Cook a message's Subject header.
+Also do other manipulations of From:, Reply-To: and Cc: depending on
+list configuration.
+"""
 
 from __future__ import nested_scopes
 import re
@@ -26,12 +29,13 @@ from email.Header import Header, decode_header, make_header
 from email.Utils import parseaddr, formataddr, getaddresses
 from email.Errors import HeaderParseError
 
+from Mailman import i18n
 from Mailman import mm_cfg
 from Mailman import Utils
 from Mailman.i18n import _
 from Mailman.Logging.Syslog import syslog
 
-CONTINUATION = ',\n\t'
+CONTINUATION = ',\n '
 COMMASPACE = ', '
 MAXLINELEN = 78
 
@@ -49,7 +53,7 @@ def _isunicode(s):
 
 nonascii = re.compile('[^\s!-~]')
 
-def uheader(mlist, s, header_name=None, continuation_ws='\t', maxlinelen=None):
+def uheader(mlist, s, header_name=None, continuation_ws=' ', maxlinelen=None):
     # Get the charset to encode the string in. Then search if there is any
     # non-ascii character is in the string. If there is and the charset is
     # us-ascii then we use iso-8859-1 instead. If the string is ascii only
@@ -62,13 +66,23 @@ def uheader(mlist, s, header_name=None, continuation_ws='\t', maxlinelen=None):
     else:
         # there is no nonascii so ...
         charset = 'us-ascii'
-    return Header(s, charset, maxlinelen, header_name, continuation_ws)
+    try:
+        return Header(s, charset, maxlinelen, header_name, continuation_ws)
+    except UnicodeError:
+        syslog('error', 'list: %s: can\'t decode "%s" as %s',
+               mlist.internal_name(), s, charset)
+        return Header('', charset, maxlinelen, header_name, continuation_ws)
 
 def change_header(name, value, mlist, msg, msgdata, delete=True, repl=True):
     if ((msgdata.get('from_is_list') == 2 or
         (msgdata.get('from_is_list') == 0 and mlist.from_is_list == 2)) and 
         not msgdata.get('_fasttrack')
-       ) or name.lower() in ('from', 'reply-to'):
+       ) or name.lower() in ('from', 'reply-to', 'cc'):
+        # The or name.lower() in ... above is because when we are munging
+        # the From:, we want to defer the resultant changes to From:,
+        # Reply-To:, and/or Cc: until after the message passes through
+        # ToDigest, ToArchive and ToUsenet.  Thus, we put them in
+        # msgdata[add_header] here and apply them in WrapMessage.
         msgdata.setdefault('add_header', {})[name] = value
     elif repl or not msg.has_key(name):
         if delete:
@@ -85,7 +99,9 @@ def process(mlist, msg, msgdata):
     # message, we want to save some of the information in the msgdata
     # dictionary for later.  Specifically, the sender header will get waxed,
     # but we need it for the Acknowledge module later.
-    msgdata['original_sender'] = msg.get_sender()
+    # We may have already saved it; if so, don't clobber it here.
+    if 'original_sender' not in msgdata:
+        msgdata['original_sender'] = msg.get_sender()
     # VirginRunner sets _fasttrack for internally crafted messages.
     fasttrack = msgdata.get('_fasttrack')
     if not msgdata.get('isdigest') and not fasttrack:
@@ -120,7 +136,18 @@ def process(mlist, msg, msgdata):
                   mlist, msg, msgdata, repl=False)
     # Do we change the from so the list takes ownership of the email
     if (msgdata.get('from_is_list') or mlist.from_is_list) and not fasttrack:
-        realname, email = parseaddr(msg['from'])
+        # Be as robust as possible here.
+        faddrs = getaddresses(msg.get_all('from', []))
+        # Strip the nulls and bad emails.
+        faddrs = [x for x in faddrs if x[1].find('@') > 0]
+        if len(faddrs) == 1:
+            realname, email = o_from = faddrs[0]
+        else:
+            # No From: or multiple addresses.  Just punt and take
+            # the get_sender result.
+            realname = ''
+            email = msgdata['original_sender']
+            o_from = (realname, email)
         if not realname:
             if mlist.isMember(email):
                 realname = mlist.getMemberName(email) or email
@@ -128,15 +155,38 @@ def process(mlist, msg, msgdata):
                 realname = email
         # Remove domain from realname if it looks like an email address
         realname = re.sub(r'@([^ .]+\.)+[^ .]+$', '---', realname)
-        # Remember the original From: here for adding to Reply-To: below.
-        o_from = parseaddr(msg['from'])
+        # Make a display name and RFC 2047 encode it if necessary.  This is
+        # difficult and kludgy. If the realname came from From: it should be
+        # ascii or RFC 2047 encoded. If it came from the list, it should be
+        # in the charset of the list's preferred language or possibly unicode.
+        # if it's from the email address, it should be ascii. In any case,
+        # make it a unicode.
+        if isinstance(realname, unicode):
+            urn = realname
+        else:
+            rn, cs = ch_oneline(realname)
+            urn = unicode(rn, cs, errors='replace')
+        # likewise, the list's real_name which should be ascii, but use the
+        # charset of the list's preferred_language which should be a superset.
+        lcs = Utils.GetCharSet(mlist.preferred_language)
+        ulrn = unicode(mlist.real_name, lcs, errors='replace')
+        # get translated 'via' with dummy replacements
+        realname = '%(realname)s'
+        lrn = '%(lrn)s'
+        # We want the i18n context to be the list's preferred_language.  It
+        # could be the poster's.
+        otrans = i18n.get_translation()
+        i18n.set_language(mlist.preferred_language)
+        via = _('%(realname)s via %(lrn)s')
+        i18n.set_translation(otrans)
+        uvia = unicode(via, lcs, errors='replace')
+        # Replace the dummy replacements.
+        uvia = re.sub(u'%\(lrn\)s', ulrn, re.sub(u'%\(realname\)s', urn, uvia))
+        # And get an RFC 2047 encoded header string.
+        dn = str(Header(uvia, lcs))
         change_header('From',
-                      formataddr(('%s via %s' % (realname, mlist.real_name),
-                                 mlist.GetListEmail())),
+                      formataddr((dn, mlist.GetListEmail())),
                       mlist, msg, msgdata)
-        if mlist.from_is_list != 2:
-            del msg['sender']
-        #MAS ?? mlist.include_sender_header = 0
     else:
         # Use this as a flag
         o_from = None
@@ -148,6 +198,23 @@ def process(mlist, msg, msgdata):
     # augment it.  RFC 2822 allows max one Reply-To: header so collapse them
     # if we're adding a value, otherwise don't touch it.  (Should we collapse
     # in all cases?)
+    # MAS: We need to do some things with the original From: if we've munged
+    # it for DMARC mitigation.  We have goals for this process which are
+    # not completely compatible, so we do the best we can.  Our goals are:
+    # 1) as long as the list is not anonymous, the original From: address
+    #    should be obviously exposed, i.e. not just in a header that MUAs
+    #    don't display.
+    # 2) the original From: address should not be in a comment or display
+    #    name in the new From: because it is claimed that multiple domains
+    #    in any fields in From: are indicative of spamminess.  This means
+    #    it should be in Reply-To: or Cc:.
+    # 3) the behavior of an MUA doing a 'reply' or 'reply all' should be
+    #    consistent regardless of whether or not the From: is munged.
+    # Goal 3) implies sometimes the original From: should be in Reply-To:
+    # and sometimes in Cc:, and even so, this goal won't be achieved in
+    # all cases with all MUAs.  In cases of conflict, the above ordering of
+    # goals is priority order.
+
     if not fasttrack:
         # A convenience function, requires nested scopes.  pair is (name, addr)
         new = []
@@ -165,13 +232,29 @@ def process(mlist, msg, msgdata):
         # the original Reply-To:'s to the list we're building up.  In both
         # cases we'll zap the existing field because RFC 2822 says max one is
         # allowed.
+        o_rt = False
         if not mlist.first_strip_reply_to:
             orig = msg.get_all('reply-to', [])
             for pair in getaddresses(orig):
+                # There's an original Reply-To: and we're not removing it.
                 add(pair)
-        # We also need to put the old From: in Reply-To: in all cases.
-        if o_from:
-            add(o_from)
+                o_rt = True
+        # We also need to put the old From: in Reply-To: in all cases where
+        # it is not going in Cc:.  This is when reply_goes_to_list == 0 and
+        # either there was no original Reply-To: or we stripped it.
+        # However, if there was an original Reply-To:, unstripped, and it
+        # contained the original From: address we need to flag that it's
+        # there so we don't add the original From: to Cc:
+        if o_from and mlist.reply_goes_to_list == 0:
+            if o_rt:
+                if d.has_key(o_from[1].lower()):
+                    # Original From: address is in original Reply-To:.
+                    # Pretend we added it.
+                    o_from = None
+            else:
+                add(o_from)
+                # Flag that we added it.
+                o_from = None
         # Set Reply-To: header to point back to this list.  Add this last
         # because some folks think that some MUAs make it easier to delete
         # addresses from the right than from the left.
@@ -200,13 +283,19 @@ def process(mlist, msg, msgdata):
         # because even though the list address is in From:, the Reply-To:
         # poster will override it. Brain dead MUAs may then address the list
         # twice on a 'reply all', but reasonable MUAs should do the right
-        # thing.
-        if (mlist.personalize == 2 and mlist.reply_goes_to_list <> 1 and
-            not mlist.anonymous_list):
+        # thing.  We also add the original From: to Cc: if it wasn't added
+        # to Reply-To:
+        add_list = (mlist.personalize == 2 and
+                    mlist.reply_goes_to_list <> 1 and
+                    not mlist.anonymous_list)
+        if add_list or o_from:
             # Watch out for existing Cc headers, merge, and remove dups.  Note
             # that RFC 2822 says only zero or one Cc header is allowed.
             new = []
             d = {}
+            # If we're adding the original From:, add it first.
+            if o_from:
+                add(o_from)
             # AvoidDuplicates may have set a new Cc: in msgdata.add_header,
             # so check that.
             if (msgdata.has_key('add_header') and
@@ -216,8 +305,9 @@ def process(mlist, msg, msgdata):
             else:
                 for pair in getaddresses(msg.get_all('cc', [])):
                     add(pair)
-            i18ndesc = uheader(mlist, mlist.description, 'Cc')
-            add((str(i18ndesc), mlist.GetListEmail()))
+            if add_list:
+                i18ndesc = uheader(mlist, mlist.description, 'Cc')
+                add((str(i18ndesc), mlist.GetListEmail()))
             change_header('Cc',
                           COMMASPACE.join([formataddr(pair) for pair in new]),
                           mlist, msg, msgdata)
@@ -291,7 +381,7 @@ def prefix_subject(mlist, msg, msgdata):
         lines = str(subject).splitlines()
     else:
         lines = subject.splitlines()
-    ws = '\t'
+    ws = ' '
     if len(lines) > 1 and lines[1] and lines[1][0] in ' \t':
         ws = lines[1][0]
     msgdata['origsubj'] = subject
@@ -321,16 +411,29 @@ def prefix_subject(mlist, msg, msgdata):
     else:
         old_style = mm_cfg.OLD_STYLE_PREFIXING
     subject = re.sub(prefix_pattern, '', subject)
-    rematch = re.match('((RE|AW|SV|VS)\s*(\[\d+\])?\s*:\s*)+', subject, re.I)
+    # Previously the following re didn't have the first \s*. It would fail
+    # if the incoming Subject: was like '[prefix] Re: Re: Re:' because of the
+    # leading space after stripping the prefix. It is not known what MUA would
+    # create such a Subject:, but the issue was reported.
+    rematch = re.match(
+                       '(\s*(RE|AW|SV|VS)\s*(\[\d+\])?\s*:\s*)+',
+                        subject, re.I)
     if rematch:
         subject = subject[rematch.end():]
         recolon = 'Re:'
     else:
         recolon = ''
+    # Strip leading and trailing whitespace from subject.
+    subject = subject.strip()
     # At this point, subject may become null if someone post mail with
-    # subject: [subject prefix]
-    if subject.strip() == '':
+    # Subject: [subject prefix]
+    if subject == '':
+        # We want the i18n context to be the list's preferred_language.  It
+        # could be the poster's.
+        otrans = i18n.get_translation()
+        i18n.set_language(mlist.preferred_language)
         subject = _('(no subject)')
+        i18n.set_translation(otrans)
         cset = Utils.GetCharSet(mlist.preferred_language)
         subject = unicode(subject, cset)
     # and substitute %d in prefix with post_id
@@ -360,6 +463,11 @@ def prefix_subject(mlist, msg, msgdata):
         except UnicodeError:
             pass
     # Get the header as a Header instance, with proper unicode conversion
+    # Because of rfc2047 encoding, spaces between encoded words can be
+    # insignificant, so we need to append spaces to our encoded stuff.
+    prefix += ' '
+    if recolon:
+        recolon += ' '
     if old_style:
         h = uheader(mlist, recolon, 'Subject', continuation_ws=ws)
         h.append(prefix)
