@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2015 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@ TBD: This needs to be made more configurable and robust.
 
 import re
 
+from unicodedata import normalize
 from email.Errors import HeaderParseError
 from email.Header import decode_header
 from email.Utils import parseaddr
@@ -36,6 +37,7 @@ from Mailman import Errors
 from Mailman import i18n
 from Mailman import Utils
 from Mailman.Handlers.Hold import hold_for_approval
+from Mailman.Logging.Syslog import syslog
 
 try:
     True, False
@@ -54,7 +56,12 @@ class SpamDetected(Errors.DiscardMessage):
     """The message contains known spam"""
 
 class HeaderMatchHold(Errors.HoldMessage):
-    reason = _('The message headers matched a filter rule')
+    def __init__(self, pattern):
+        self.__pattern = pattern
+
+    def reason_notice(self):
+        pattern = self.__pattern
+        return _('Header matched regexp: %(pattern)s')
 
 
 # And reset the translator
@@ -63,11 +70,11 @@ _ = i18n._
 
 
 def getDecodedHeaders(msg, cset='utf-8'):
-    """Returns a string containing all the headers of msg, unfolded and
-    RFC 2047 decoded and encoded in cset.
+    """Returns a unicode containing all the headers of msg, unfolded and
+    RFC 2047 decoded, normalized and separated by new lines.
     """
 
-    headers = ''
+    headers = u''
     for h, v in msg.items():
         uvalue = u''
         try:
@@ -86,7 +93,8 @@ def getDecodedHeaders(msg, cset='utf-8'):
                 # unicode it as iso-8859-1 which may result in a garbled
                 # mess, but we have to do something.
                 uvalue += unicode(frag, 'iso-8859-1', 'replace')
-        headers += '%s: %s\n' % (h, uvalue.encode(cset, 'replace'))
+        uhdr = h.decode('us-ascii', 'replace')
+        headers += u'%s: %s\n' % (h, normalize(mm_cfg.NORMALIZE_FORM, uvalue))
     return headers
 
 
@@ -114,6 +122,7 @@ def process(mlist, msg, msgdata):
                     if text:
                         text = Utils.wrap(text)
                     else:
+                        listowner = mlist.GetOwnerEmail()
                         text = Utils.wrap(_(
 """You are not allowed to post to this mailing list From: a domain which
 publishes a DMARC policy of reject or quarantine, and your message has been
@@ -122,6 +131,21 @@ error, contact the mailing list owner at %(listowner)s."""))
                     raise Errors.RejectMessage, text
                 elif mlist.dmarc_moderation_action == 4:
                     raise Errors.DiscardMessage
+
+        # Get member address if any.
+        for sender in msg.get_senders():
+            if mlist.isMember(sender):
+                break
+        else:
+            sender = msg.get_sender()
+        if (mlist.member_verbosity_threshold > 0 and
+            Utils.IsVerboseMember(mlist, sender)
+           ):
+             mlist.setMemberOption(sender, mm_cfg.Moderate, 1)
+             syslog('vette',
+                    '%s: Automatically Moderated %s for verbose postings.',
+                     mlist.real_name, sender) 
+
     if msgdata.get('approved'):
         return
     # First do site hard coded header spam checks
@@ -135,7 +159,7 @@ error, contact the mailing list owner at %(listowner)s."""))
     # Now do header_filter_rules
     # TK: Collect headers in sub-parts because attachment filename
     # extension may be a clue to possible virus/spam.
-    headers = ''
+    headers = u''
     # Get the character set of the lists preferred language for headers
     lcset = Utils.GetCharSet(mlist.preferred_language)
     for p in msg.walk():
@@ -149,7 +173,17 @@ error, contact the mailing list owner at %(listowner)s."""))
             # ignore 'empty' patterns
             if not pattern.strip():
                 continue
-            if re.search(pattern, headers, re.IGNORECASE|re.MULTILINE):
+            pattern = Utils.xml_to_unicode(pattern, lcset)
+            pattern = normalize(mm_cfg.NORMALIZE_FORM, pattern)
+            try:
+                mo = re.search(pattern,
+                               headers,
+                               re.IGNORECASE|re.MULTILINE|re.UNICODE)
+            except (re.error, TypeError):
+                syslog('error',
+                       'ignoring header_filter_rules invalid pattern: %s',
+                       pattern)
+            if mo:
                 if action == mm_cfg.DISCARD:
                     raise Errors.DiscardMessage
                 if action == mm_cfg.REJECT:
@@ -166,6 +200,7 @@ error, contact the mailing list owner at %(listowner)s."""))
                         # pass it here but list-owner can set this to be
                         # discarded on the GUI if he wants.
                         return
-                    hold_for_approval(mlist, msg, msgdata, HeaderMatchHold)
+                    hold_for_approval(
+                        mlist, msg, msgdata, HeaderMatchHold(pattern))
                 if action == mm_cfg.ACCEPT:
                     return
