@@ -1,4 +1,4 @@
-# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,12 +12,15 @@
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software 
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import os
 import time
-from Logger import Logger
 
+from Mailman.Logging.Logger import Logger
+
+
+
 class StampedLogger(Logger):
     """Record messages in log files, including date stamp and optional label.
 
@@ -62,7 +65,7 @@ class StampedLogger(Logger):
                 self.__primed = 0
             else:
                 stamp = ""
-            if self.__label == None:
+            if self.__label is None:
                 label = "(%d)" % os.getpid()
             else:
                 label = "%s(%d):" % (self.__label, os.getpid())
